function X=metrics4D(seg,truth)
% function X=metrics4D(segnames,truthnames)
% SEGNAMES is a string containing automatic segmentation file names, COMMA
% SEPARATED
% TRUTHNAMES is a string containing manual segmentation file names, COMMA
% SEPARATED
% All files must be NIFTI nii
% Ouput is a structure containing two fields,
%   threeDmetrics -- See metrics3D.m
%   fourDmetrics 
%       LTPR/LFPR : lesion TPR/FPR based on "new" lesion count
% LTPR(LFPR) is a T-1 vector, containing pairwise detection or false
% positive rates of new (disappearing from first time-point appearing 
% in next time-point) lesions. T is the total number of time-points
% If there are no new lesions between a pair, the code will return NaN


temp=strsplit(seg,',');
T=length(temp);
segnames=cell(T,1);
fprintf('%d automatic segmentation images detected.\n',T);
for t=1:T
    segnames{t}=temp{t};
    fprintf('%s\n',extract_name(segnames{t}));
end

temp=strsplit(truth,',');
T=length(temp);
truthnames=cell(T,1);
fprintf('%d manual segmentation images detected.\n',T);
for t=1:T
    truthnames{t}=temp{t};
    fprintf('%s\n',extract_name(truthnames{t}));
end

conn=18;
if length(segnames)~=length(truthnames)
    fprintf('Number of volumes must be same, %d vs %s\n',length(segnames),length(truthnames));
    X=[];
end
L=length(segnames);
X.threeDmetrics=cell(L,1);
X.fourDmetrics.LTPR=[];
X.fourDmetrics.LFPR=[];


NumCPU=10;
if feature('NumCores')<NumCPU NumCPU=feature('NumCores'); end;
if matlabpool('size')<NumCPU
    try
    matlabpool close;
    end
%     fprintf('Opening %d new matlabpools.\n',NumCPU);
    c=parcluster('local');c.NumWorkers=NumCPU;
    matlabpool open;
    flag=1;
else
    flag=0;
%     fprintf('using existing %d open matlabpools.\n',matlabpool('size'));
end

for t=1:L
    fprintf('Time-point %d -- \n',t);
    y=metrics3D(segnames{t},truthnames{t});
    X.threeDmetrics{t}=y;
end

for t=1:L-1
    temp1=load_untouch_nii(truthnames{t});temp1=temp1.img;temp11=temp1;
    temp2=load_untouch_nii(truthnames{t+1});temp2=temp2.img;temp22=temp2;
    cc1=bwconncomp(temp11,conn);
    cc2=bwconncomp(temp22,conn);    
%     Keep only those lesions in 1st time-point that disappeared on 2nd time-point
    indx=find(temp22==1);
    for i=1:cc1.NumObjects
        if ~isempty(intersect(cc1.PixelIdxList{i},indx))
            temp1(cc1.PixelIdxList{i})=0;
        end
    end
%     Keep only those lesions in 2nd time-point that appeared on 2nd time-point
    indx=find(temp11==1);
    for i=1:cc2.NumObjects
        if ~isempty(intersect(cc2.PixelIdxList{i},indx))
            temp2(cc2.PixelIdxList{i})=0;
        end
    end
    
%     Find LTPR on only changing lesions on truth
    seg1=load_untouch_nii(segnames{t});seg1=seg1.img;seg11=seg1;
    seg2=load_untouch_nii(segnames{t+1});seg2=seg2.img;seg22=seg2;
    if sum(temp1(:))>0 && sum(temp2(:))>0
        X.fourDmetrics.LTPR(t)=(LTPR(seg1,temp1,conn)+LTPR(seg2,temp2,conn))/2;
    elseif sum(temp1(:))==0 && sum(temp2(:))>0
        X.fourDmetrics.LTPR(t)=LTPR(seg2,temp2,conn);
    elseif sum(temp1(:))>0 && sum(temp2(:))==0
        X.fourDmetrics.LTPR(t)=LTPR(seg1,temp1,conn);
    else
        X.fourDmetrics.LTPR(t)=NaN;
    end


    cc1=bwconncomp(seg11,conn);
    cc2=bwconncomp(seg22,conn);    
%     Keep only those lesions in 1st time-point that disappeared in 2nd time-point
    indx=find(seg22==1);
    for i=1:cc1.NumObjects
        if ~isempty(intersect(cc1.PixelIdxList{i},indx))
            seg1(cc1.PixelIdxList{i})=0;
        end
    end
%     Keep only those lesions in 2nd time-point that appeared in 2nd time-point
    indx=find(seg11==1);
    for i=1:cc2.NumObjects
        if ~isempty(intersect(cc2.PixelIdxList{i},indx))
            seg2(cc2.PixelIdxList{i})=0;
        end
    end
    
    %     Find LFPR on only changing lesions in seg
    temp1=load_untouch_nii(truthnames{t});temp1=temp1.img;
    temp2=load_untouch_nii(truthnames{t+1});temp2=temp2.img;
    if sum(seg1(:))>0 && sum(seg2(:))>0
        X.fourDmetrics.LFPR(t)=(LFPR(seg1,temp1,conn)+LFPR(seg2,temp2,conn))/2;
    elseif sum(seg1(:))==0 && sum(seg2(:))>0
        X.fourDmetrics.LFPR(t)=LFPR(seg2,temp2,conn);
    elseif sum(seg1(:))>0 && sum(seg2(:))==0
        X.fourDmetrics.LFPR(t)=LFPR(seg1,temp1,conn);
    else
        X.fourDmetrics.LFPR(t)=NaN;
    end
    
end

if flag
    matlabpool close;
end

Vt=zeros(L,1);Vs=zeros(L,1);
for t=1:L Vt(t)=X.threeDmetrics{t}.ManualVol; end;
for t=1:L Vs(t)=X.threeDmetrics{t}.SegVol; end;
tmpT=zeros(L-1,1);tmpS=zeros(L-1,1);
for t=1:L-1
    tmpT(t)=Vt(t+1)-Vt(t);
    tmpS(t)=Vs(t+1)-Vs(t);
end
X.fourDmetrics.Corr=corr(tmpS,tmpT);




fprintf('---------------------------------------------------\n')
fprintf('Time-point,Dice,Jaccard,PPV,Sensititvity,Lesion TPR,Lesion FPR,');
fprintf('Volume Difference,Surface Difference,Segmentation Volume,Manual Volume,');
fprintf('Volume Change Correlation,New Lesion detection TPR,New Lesion detection FPR\n');
for t=1:L-1
    x=X.threeDmetrics{t};
    fprintf('%d,%.5f,%.5f,%.5f,%.5f,%.5f,%.5f,%.5f,%.5f,',...
        t,x.Dice,x.Jaccard,x.PPV,x.TPR,x.LTPR,x.LFPR,x.VD,x.SD);
    y=X.fourDmetrics;
    fprintf('%.2f,%.2f,%.5f,%.5f,%.5f\n',x.SegVol,x.ManualVol,y.Corr,y.LTPR(t),y.LFPR(t));
end
x=X.threeDmetrics{T};
fprintf('%d,%.5f,%.5f,%.5f,%.5f,%.5f,%.5f,%.5f,%.5f,',...
    T,x.Dice,x.Jaccard,x.PPV,x.TPR,x.LTPR,x.LFPR,x.VD,x.SD);
fprintf('%.2f,%.2f,%.5f\n',x.SegVol,x.ManualVol,y.Corr);

fprintf('---------------------------------------------------\n')    



function x=LTPR(seg,truth,conn)
cc=bwconncomp(truth,conn);
indx=find(seg==1);
count=0;
for t=1:cc.NumObjects
    if ~isempty(intersect(cc.PixelIdxList{t},indx))       
        count=count+1;
    end
end
x=count/cc.NumObjects;


function x=LFPR(seg,truth,conn)
cc=bwconncomp(seg,conn);
indx=find(truth==1);
count=0;
for t=1:cc.NumObjects
     if isempty(intersect(cc.PixelIdxList{t},indx))
         count=count+1;
     end
end
x=count/cc.NumObjects;

