#!/bin/sh
if [ $# -lt "2" ]; then
echo "
Usage: ./metrics4D.sh SEGFILENAMES TRUTHFILENAMES
SEGFILENAMES    Full path (NO relative path) of the automatic binary segmentation volumes,
                COMMA separated. (Must be NIFTI nii files, NIFTI.GZ not acceptable)
                E.g. /home/user/seg1.nii,/home/user/seg2.nii
TRUTHFILENAMES  Full path (NO relative path) of the manual binary segmentation volumes,
                COMMA separated. 
                E.g. /home/user/manualseg1.nii,/home/user/manualseg2.nii

The code computes the following metrics,
     Dice
     Jaccard (Volume overlap or VO)
     PPV (positive predictive value)
     TPR (sensitivity, voxel based)
     LTPR (lesion TPR based on lesion count)
     LFPR (lesion FPR based on lesion count)
     VD (absolute volume difference)
     SD (average absolute symmetric surface distance)
     Volume difference correlation 
     New lesion detection TPR (# of new lesions detected/ # of true new lesions in the manual segmentation)
     New lesion detection FPR (# of new lesions falsely appeared in the automatic segmentation/ # of new lesions in the automatic segmentation)

NOTE: If there are no new lesions, the code will return NaN

Some definitions are based on the following paper
http://www.sciencedirect.com/science/article/pii/S1053811911003740   

Change path of MCRROOT (Matlab 2013a v81) if necessary

"
exit 1
fi
SEG=$1
MAN=$2
exe_name=$0
exe_dir=`dirname "$0"`
 
  MCRROOT=/home/roy/Matlab_Compiler_Runtime/v81
  echo MCRROOT is $MCRROOT
  LD_LIBRARY_PATH=.:${MCRROOT}/runtime/glnxa64 ;
  LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${MCRROOT}/bin/glnxa64 ;
  LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${MCRROOT}/sys/os/glnxa64;
	MCRJRE=${MCRROOT}/sys/java/jre/glnxa64/jre/lib/amd64 ;
	LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${MCRJRE}/native_threads ; 
	LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${MCRJRE}/server ;
	LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${MCRJRE}/client ;
	LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${MCRJRE} ;  
  XAPPLRESDIR=${MCRROOT}/X11/app-defaults ;
  export LD_LIBRARY_PATH;
  export XAPPLRESDIR;  
  shift 1  
  ${exe_dir}/metrics4D $SEG $MAN


